package example.account;

import java.util.ArrayList;
import java.util.List;

/**
 * Account implementation.
 */
public class AccountImpl implements Account {

    private String id;
    private String ownerName;
    private int balance;
    private List<Transaction> transactions;

    private Supervisor supervisor;
    public static final int SUPERVISION_TRESHOLD = 100000;

    /**
     * Create an AccountImpl with and id and owner.
     * 
     * @param id
     *            the account id
     * @param owner
     *            the owner
     */
    public AccountImpl(String id, String owner) {
        this.id = id;
        this.ownerName = owner;
        this.balance = 0;
        this.transactions = new ArrayList<Transaction>();
    }

    /**
     * Deposit a specified amount to the account. Updates the balance
     * correspondingly, and adds a corresponding Transaction to the Account.
     * 
     * @param amt
     *            the amount to deposit.
     * @see Transaction
     * @see example.account.Account#deposit(double)
     */
    public void deposit(int amt) {
        Transaction tx = new Transaction(Transaction.DEPOSIT, amt);
        if (amt >= SUPERVISION_TRESHOLD && supervisor != null) {
            if (!supervisor.notify(this.id, this.ownerName, tx)) {
                throw new SupervisorException(
                        "Transaction not allowed by supervisor");
            }
        }
        balance += amt;
        transactions.add(tx);
    }

    /**
     * Withdraw a specified amount from the account. Updates the balance
     * correspondingly, and adds a corresponding Transaction to the Account.
     * 
     * @param amt
     *            the amount to withdraw.
     * @throws AccountException
     *             if not enough funding exists.
     * @see Transaction
     * @see example.account.Account#withdraw(double)
     */
    public void withdraw(int amt) throws AccountException {
        if (amt <= balance) {
            Transaction tx = new Transaction(Transaction.WITHDRAW, amt);
            if (amt >= SUPERVISION_TRESHOLD && supervisor != null) {
                if (!supervisor.notify(this.id, this.ownerName, tx)) {
                    throw new SupervisorException(
                            "Transaction not allowed by supervisor");
                }
            }
            balance -= amt;
            transactions.add(tx);
        } else {
            throw new AccountException("Balance exceeded!");
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see example.account.Account#getBalance()
     */
    public int getBalance() {
        return balance;
    }

    /*
     * (non-Javadoc)
     * 
     * @see example.account.Account#getOwnerName()
     */
    public String getOwnerName() {
        return this.ownerName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see example.account.Account#setOwnerName(java.lang.String)
     */
    public void setOwnerName(String name) {
        this.ownerName = name;
    }

    /*
     * (non-Javadoc)
     * 
     * @see example.account.Account#getAccountID()
     */
    public String getAccountID() {
        return this.id;
    }

    /*
     * (non-Javadoc)
     * 
     * @see example.account.Account#setAccountID(java.lang.String)
     */
    protected void setAccountID(String id) {
        this.id = id;
    }

    /*
     * (non-Javadoc)
     * 
     * @see example.account.Account#getTransctions()
     */
    public List<Transaction> getTransctions() {
        return new ArrayList<Transaction>(transactions);
    }

    public void setSupervisor(Supervisor supervisor) {
        this.supervisor = supervisor;
    }

}
