package example.account;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

public class AccountImplTest {

	private AccountImpl account;

    @Before
    public void setUp() {
        account = new AccountImpl("testId", "testOwner");
    }

	@Test
	public void testInitialBalance() {
        account = new AccountImpl("testId", "testOwner");
        int expectedBalance = 0;
        int actualBalance = account.getBalance();
        Assert.assertEquals(expectedBalance, actualBalance);
    }
	
    @Test
    public void testWithdraw() throws AccountException {
        int amount = 100;
 
        account.deposit(2 * amount);
        account.withdraw(amount);

        int expectedBalance = amount;
        int actualBalance = account.getBalance();
        Assert.assertEquals(expectedBalance, actualBalance);
    }

    @Test
    public void testDeposit() {
        int amount = 100;
        account.deposit(amount);

        int expectedBalance = amount;
        int actualBalance = account.getBalance();
        Assert.assertEquals(expectedBalance, actualBalance);
    }

    @Test
    public void testDepositTwice() {
        int amount = 100;
        account.deposit(amount);
        account.deposit(amount);

        int expectedBalance = 2 * amount;
        int actualBalance = account.getBalance();
        Assert.assertEquals(expectedBalance, actualBalance);
    }

 
    @Test
    public void testWithdrawAll() throws AccountException {
        int amount = 100;
        account.deposit(amount);

        account.withdraw(amount);

        int expectedBalance = 0;
        int actualBalance = account.getBalance();
        Assert.assertEquals(expectedBalance, actualBalance);
    }

    @Test
    public void testWithdrawTooMuch() throws AccountException {
        int amount = 100;
        account.deposit(amount);

        try {
            account.withdraw(amount + 1);
            Assert.fail("AccountException expected");
        } catch (AccountException e) {
            // Expected
        }
        int expectedBalance = amount;
        int actualBalance = account.getBalance();
        Assert.assertEquals(expectedBalance, actualBalance);
    }

    @Test
    public void testLargeDepositShouldTriggerNotification() {
        int amount = AccountImpl.SUPERVISION_TRESHOLD;
        Supervisor mockSupervisor = Mockito.mock(Supervisor.class);
        Mockito.when(
            mockSupervisor.notify(Mockito.anyString(), Mockito.anyString(), (Transaction) Mockito.anyObject()))
            .thenReturn(true);
        account.setSupervisor(mockSupervisor);

        account.deposit(amount);

        Mockito.verify(mockSupervisor).notify(account.getAccountID(), account.getOwnerName(),
            new Transaction(Transaction.DEPOSIT, amount));
        Mockito.verifyNoMoreInteractions(mockSupervisor);
    }

    @Test
    public void testSmallDepositShouldNotTriggerNotification() {
        int amount = AccountImpl.SUPERVISION_TRESHOLD - 1;
        Supervisor mockSupervisor = Mockito.mock(Supervisor.class);
        account.setSupervisor(mockSupervisor);

        account.deposit(amount);

        Mockito.verifyZeroInteractions(mockSupervisor);
    }

    @Test
    public void testNotificationVetoShouldBeHonoured() {
        int amount = AccountImpl.SUPERVISION_TRESHOLD;
        Supervisor mockSupervisor = Mockito.mock(Supervisor.class);
        // Default boolean stub return value is false, hence the following stub instruction is redundant
        Mockito.when(
            mockSupervisor.notify(Mockito.anyString(), Mockito.anyString(), (Transaction) Mockito.anyObject()))
            .thenReturn(false);
        account.setSupervisor(mockSupervisor);

        try {
            account.deposit(amount);
            Assert.fail("SupervisorException expected");
        } catch (SupervisorException expected) {
            // expected
            System.err.println(expected);
        }
        Mockito.verify(mockSupervisor).notify(account.getAccountID(), account.getOwnerName(),
            new Transaction(Transaction.DEPOSIT, amount));
    }

}    
/*
    private AccountImpl account;

    @Before
    public void setUp() {
        account = new AccountImpl("testId", "testOwner");
    }

    @Test
    public void testInitialBalance() {
        account = new AccountImpl("testId", "testOwner");
        int expectedBalance = 0;
        int actualBalance = account.getBalance();
        Assert.assertEquals(expectedBalance, actualBalance);
    }

    @Test
    public void testDeposit() {
        int amount = 100;
        account.deposit(amount);

        int expectedBalance = amount;
        int actualBalance = account.getBalance();
        Assert.assertEquals(expectedBalance, actualBalance);
    }

    @Test
    public void testDepositTwice() {
        int amount = 100;
        account.deposit(amount);
        account.deposit(amount);

        int expectedBalance = 2 * amount;
        int actualBalance = account.getBalance();
        Assert.assertEquals(expectedBalance, actualBalance);
    }

    @Test
    public void testWithdraw() throws AccountException {
        int amount = 100;
        account.deposit(2 * amount);

        account.withdraw(amount);

        int expectedBalance = amount;
        int actualBalance = account.getBalance();
        Assert.assertEquals(expectedBalance, actualBalance);
    }

    @Test
    public void testWithdrawAll() throws AccountException {
        int amount = 100;
        account.deposit(amount);

        account.withdraw(amount);

        int expectedBalance = 0;
        int actualBalance = account.getBalance();
        Assert.assertEquals(expectedBalance, actualBalance);
    }

    @Test
    public void testWithdrawTooMuch() throws AccountException {
        int amount = 100;
        account.deposit(amount);

        try {
            account.withdraw(amount + 1);
            Assert.fail("AccountException expected");
        } catch (AccountException e) {
            // Expected
        }
        int expectedBalance = amount;
        int actualBalance = account.getBalance();
        Assert.assertEquals(expectedBalance, actualBalance);
    }

    @Test
    public void testLargeDepositShouldTriggerNotification() {
        int amount = AccountImpl.SUPERVISION_TRESHOLD;
        Supervisor mockSupervisor = Mockito.mock(Supervisor.class);
        Mockito.when(
            mockSupervisor.notify(Mockito.anyString(), Mockito.anyString(), (Transaction) Mockito.anyObject()))
            .thenReturn(true);
        account.setSupervisor(mockSupervisor);

        account.deposit(amount);

        Mockito.verify(mockSupervisor).notify(account.getAccountID(), account.getOwnerName(),
            new Transaction(Transaction.DEPOSIT, amount));
        Mockito.verifyNoMoreInteractions(mockSupervisor);
    }

    @Test
    public void testSmallDepositShouldNotTriggerNotification() {
        int amount = AccountImpl.SUPERVISION_TRESHOLD - 1;
        Supervisor mockSupervisor = Mockito.mock(Supervisor.class);
        account.setSupervisor(mockSupervisor);

        account.deposit(amount);

        Mockito.verifyZeroInteractions(mockSupervisor);
    }

    @Test
    public void testNotificationVetoShouldBeHonoured() {
        int amount = AccountImpl.SUPERVISION_TRESHOLD;
        Supervisor mockSupervisor = Mockito.mock(Supervisor.class);
        
        // Default boolean stub return value is false, hence the following stub instruction is redundant
        Mockito.when(
            mockSupervisor.notify(Mockito.anyString(), Mockito.anyString(), (Transaction) Mockito.anyObject()))
            .thenReturn(false);
        account.setSupervisor(mockSupervisor);
        
        try {
            account.deposit(amount);
            Assert.fail("SupervisorException expected");
        } catch (SupervisorException expected) {
            // expected
            System.err.println(expected);
        }
        Mockito.verify(mockSupervisor).notify(account.getAccountID(), account.getOwnerName(),
            new Transaction(Transaction.DEPOSIT, amount));
    }
*/


